'use strict';

// 加密类
const crypto = require('crypto');
const messageType = require('../const/message_type');

module.exports = options => {
    /**
     * session判断中间件
     *
     * @param {function} next - 中间件继续执行的方法
     * @return {void}
     */
    return function* sessionAuth(next) {
        try {
            // 判断session
            const sessionUser = this.session.sessionUser;
            if (sessionUser === undefined) {
                throw '不存在session';
            }
            // 校验session
            if (sessionUser.account === undefined || sessionUser.loginTime === undefined) {
                throw '用户数据不完整';
            }
            // 校验session
            const sessionToken = crypto.createHmac('sha1', sessionUser.loginTime + '')
                .update(sessionUser.account).digest().toString('base64');
            if (sessionToken !== sessionUser.sessionToken) {
                throw 'session数据错误';
            }

            // 同步消息
            yield this.service.notify.syncNotifyData();
        } catch (error) {
            console.log(error);
            if (this.helper.isAjax(this.request)) {
                return this.body = {
                    err: 2,
                    msg: '登录信息异常，请重新登录',
                    data: '',
                };
            } else {
                this.session.message = {
                    type: messageType.ERROR,
                    icon: 'exclamation-circle',
                    message: '登录信息异常，请重新登录',
                };
                return this.redirect('/login?referer=' + this.url);
            }
        }
        yield next;
    };
};
